'''initialize'''
from . import nn
from . import optim
from . import functional


'''package相关变量'''
__title__ = 'pytoydl'
__description__ = 'Pytoydl: A toy deep learning framework built upon numpy.'
__url__ = 'https://github.com/CharlesPikachu/pytoydl'
__version__ = '0.0.1'
__author__ = 'Zhenchao Jin'
__email__ = 'charlesblwx@gmail.com'
__license__ = 'Apache License 2.0'
__copyright__ = 'Copyright 2022 Zhenchao Jin'